@@ -2,7 +2,7 @@ package Net::Amazon::DynamoDB::Marshaler;
 
 use strict;
 use 5.008_005;
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use parent qw(Exporter);
 our @EXPORT = qw(dynamodb_marshal dynamodb_unmarshal);
@@ -14,33 +14,46 @@ use Types::Standard qw(StrictNum);
 sub dynamodb_marshal {
     my ($attrs, %args) = @_;
     my $force_type = $args{force_type} || {};
-    die __PACKAGE__.'dynamodb_marshal(): argument must be a hashref' unless (
-        ref $attrs
-        && ref $attrs eq 'HASH'
-    );
-    die __PACKAGE__.'dynamodb_marshal(): force_type must be a hashref' unless (
-        ref $force_type
-        && ref $force_type eq 'HASH'
-    );
+    die __PACKAGE__.'::dynamodb_marshal(): argument must be a hashref'
+        unless (
+            ref $attrs
+            && ref $attrs eq 'HASH'
+        );
+    die __PACKAGE__.'::dynamodb_marshal(): force_type must be a hashref'
+        unless (
+            ref $force_type
+            && ref $force_type eq 'HASH'
+        );
     return _marshal_hashref($attrs, $force_type);
 }
 
 sub dynamodb_unmarshal {
     my ($attrs) = @_;
-    die __PACKAGE__.'dynamodb_unmarshal(): argument must be a hashref' unless (
-        ref $attrs
-        && ref $attrs eq 'HASH'
-    );
+    die __PACKAGE__.'::dynamodb_unmarshal(): argument must be a hashref'
+        unless (
+            ref $attrs
+            && ref $attrs eq 'HASH'
+        );
     return _unmarshal_hashref($attrs);
 }
 
 sub _marshal_hashref {
     my ($attrs, $force_type) = @_;
     $force_type ||= {};
-    return {
-        map { $_ => _marshal_val($attrs->{$_}, $force_type->{$_}) }
-        keys %$attrs
-    };
+    my %marshalled;
+    for my $key (keys %$attrs) {
+        my $val = $attrs->{$key};
+        my $new_val;
+        if (my $type = $force_type->{$key}) {
+            $new_val = _marshal_val_force_type($val, $type);
+        } else {
+            $new_val = _marshal_val($val);
+        }
+        if ($new_val) {
+            $marshalled{$key} = $new_val;
+        }
+    }
+    return \%marshalled;
 }
 
 sub _unmarshal_hashref {
@@ -49,8 +62,8 @@ sub _unmarshal_hashref {
 }
 
 sub _marshal_val {
-    my ($val, $type) = @_;
-    $type ||= _val_type($val);
+    my ($val) = @_;
+    my $type = _val_type($val);
 
     return { $type => $val } if $type =~ /^(N|S)$/;
     return { $type => 1 } if $type eq 'NULL';
@@ -62,6 +75,22 @@ sub _marshal_val {
     die "don't know how to marshal type of $type";
 }
 
+sub _marshal_val_force_type {
+    my ($val, $type) = @_;
+
+    if ($type eq 'N') {
+        return undef unless StrictNum->check($val);
+        return { N => $val };
+    }
+
+    if ($type eq 'S') {
+        return undef unless (defined $val && length($val));
+        return { S => "$val" };
+    }
+
+    die __PACKAGE__.'::dynamodb_marshal(): force_type only supports "S" and "N" types';
+}
+
 sub _unmarshal_attr_val {
     my ($attr_val) = @_;
     my ($type, $val) = %$attr_val;
@@ -253,6 +282,10 @@ Use force_type in that situation:
   #   ...
   # };
 
+The module only supports 'S' and 'N' types for force_type. If you specify 'S', dynamodb_marshal will stringify the value, so make sure not to send arrays, etc. as values. If you specify 'N', dynamodb_marshal will set the value to undef if it's not a number.
+
+Undefs or empty string values for force_type attributes will be removed from the marshalled hashref. While this behavior might not seem intuitive at first, it's almost certainly what you want. For instance, if you have a global secondary index on a string attribute, and your item has an undef value for that attribute, you want to avoid sending that attribute (using NULL would be rejected by DynamoDB, and you can't send empty strings). If you have an undef value for a primary key string attribute, you have a bug in your application somewhere.
+
 =head2 dynamodb_unmarshal
 
 The opposite of dynamodb_marshal.
